;; Generated package description from gnome-dark-style.el  -*- mode: lisp-data; no-byte-compile: t; lexical-binding:t -*-
(define-package "gnome-dark-style" "0.2.4" "Sync theme with GNOME color-scheme" '((emacs "30.1")) :commit "ede0104f21ebdc0128b6cbe2c31962ab335b0ea6" :authors '(("David Dimagid" . "davidimagid@gmail.com")) :maintainer '("David Dimagid" . "davidimagid@gmail.com") :keywords '("themes" "gnome" "sync" "dark" "light" "color-scheme") :url "https://github.com/dimagid/gnome-dark-style")
